//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         jaruwit supa, Base on FontEditor written by H. Reddmann
//								 Modified by Sh. Nourbakhsh Rad for Persian and Arabic font
//								 and fixed some errors
// Remarks:        
// known Problems: none
// Version:        1.7.0  01.09.2011
// Description:    Font Library
//-----------------------------------------------------------------------------

#include "Font.h"
#include "PE_Map.h"
#include "ili9325-2.h"
#include "ili9325.h"

const unsigned char 			*FontPointer; 								// Font	Pointer
unsigned char 						rot 						= 0;					// Rot 0=0, 1=90
unsigned char 						Reverse					= 0;					// Reverse text direction on the display
unsigned char							Horizontal			=	1;					// 1: Portrait - 240x320 ,   2: Landscape - 320x240

unsigned short 						FgColor 				= BLACK;			// Text fg color, 5-6-5 RGB
unsigned short 						BkColor 				= WHITE;			// Text bk color, 5-6-5 RGB
unsigned char 						FontFixed				= 0;					// Text type 0=Proportional , 1=Fixed
unsigned char							NonTransparence = 0;					// Transparent 0=No, 1=Yes

unsigned short 						fontSize;											// size of current font
unsigned char 						firstchar;										// first character noumber of current font
unsigned char 						lastchar;											// last character noumber of current font
unsigned char 						charwidth;										// current character width register

unsigned char							FontWidth;										// max width of font
unsigned char							FontHeight;										// max height of font
unsigned char 						FontXScale 			= 1;					// X size of font
unsigned char 						FontYScale 			= 1;					// Y size of font
unsigned char 						FontSpace 			= 1;					// space between char

unsigned int 							cursorX 				= 0;					// x position
unsigned int 							cursorY 				= 0;					// y position

unsigned char							peLETTER				= E_LETTER;		// English or Persian letter

unsigned char 						prevLet 				= 0xFF;				// previous persian character register
unsigned char	 						nextLet 				= 0xFF; 			// next persian character register

#define 									FONT_HEADER_SIZE  7 					// header size of fonts

//********************************************
//********************************************

//**** select font
void LcdFont(const unsigned char *pointer)
{
	FontPointer 			= pointer;

	FontWidth 				= PROGMEM_read_b(FontPointer[2]);
	FontHeight				= PROGMEM_read_b(FontPointer[3]);

	//fontSize				=	PROGMEM_read_w(FontPointer[0]);
	fontSize					=	PROGMEM_read_b(FontPointer[1]) << 8;
	fontSize				 += PROGMEM_read_b(FontPointer[0]);
	firstchar 				=	PROGMEM_read_b(FontPointer[5]);
	lastchar 					=	PROGMEM_read_b(FontPointer[6]);
}	//LcdFont

//**** draw char
void PutChar(unsigned char c)
{
  if(peLETTER == P_LETTER)
		PutCharPE(c);
	else
			PutCharEN(c);
}	//PutChar

void SetLine(unsigned char line,unsigned char column)
{
	if(peLETTER == P_LETTER)
		cursorX =GetMaxX() - (unsigned int)FontWidth  * FontXScale * column;
	else
		cursorX =(unsigned int)FontWidth  * FontXScale * column;

	cursorY =(unsigned int)FontHeight * FontYScale * line;
}	//SetCursor

unsigned int CalcTextWidth(char *Text)
{
	if(peLETTER == E_LETTER)
		return CalcTextWidthEN(Text);
	else
		return CalcTextWidthPE(Text);
}	//CalcTextWidth

unsigned int CalcTextHeight(char *Text)
{
	return FontHeight * FontYScale;
}	//CalcTextHeight

//**** draw string
void Puts(char *Text)
{	
	unsigned char 	c;
	
  if(peLETTER == P_LETTER)
		PutsPE(Text);
	else
	{
		while((c = *Text++))
			PutCharEN(c);
	}
}	//Puts

void PutsP(char *Text)
{
	unsigned char 	c;
	
	while((c = PROGMEM_read_b(*Text++)))
	{
		PutChar(c);
	}
}	//PutsP

void PutLong(unsigned long src, unsigned char digit, unsigned char decimal, unsigned char Parameter)
{
	char 			text[10+1];
	unsigned char 	i;
	Lt_Mode		cMode;
	
	cMode = peLETTER;
	SetLetter(E_LETTER);
	
	text[digit] = 0;
	for(i = digit; i >0; i--)
	{
		text[i-1] =(src % 10) + 48;
		src = src/10;
	}//for digit
	
	if (Parameter & (STYLE_NO_ZERO | STYLE_NO_SPACE))
	{
		while(text[i] == '0')
		{
		
			if (Parameter & (STYLE_NO_ZERO))
				PutChar(' ');
			
			i++;
			if (i > digit - decimal -1 -1) // include lead 0.xx
				break;
		}
	}
	
	while(text[i])
	{
		if (i == digit - decimal) // include lead 0.xx
			PutChar('.');
		
		PutChar(text[i]);
		i++;
	}

	SetLetter(cMode);
}	//PutLong

void PutInt(unsigned short src, unsigned char digit, unsigned char decimal, unsigned char Parameter)
{
	unsigned long 		n = src;
	
	PutLong(n, digit, decimal, Parameter);
}	//PutInt

void PutByte(unsigned char src, unsigned char digit, unsigned char decimal, unsigned char Parameter)
{
	unsigned long 		n = src;
	
	PutLong(n, digit, decimal, Parameter);
}	//PutByte

void PutText(unsigned short left, unsigned short top, unsigned short right, unsigned short bottom, char *str, unsigned char style)
{
	unsigned short 			sizeofstring;

	sizeofstring = CalcTextWidth(str);
	top = top + (bottom - top - (FontHeight*FontYScale))/2;

	
	if(peLETTER == E_LETTER)
	{
		switch (style & ALINE_MARK)
		{
			case ALINE_LEFT:
				left = left + FontSpace;
				break;
	
			case ALINE_CENTER:
					left = left + (right - left - sizeofstring) / 2;
				break;
	
			case ALINE_RIGHT:
				if (right > sizeofstring)
					left = right - sizeofstring;
				break;
		}//switch

		SetCursorX(left);
	}
	else
	{
		switch (style & ALINE_MARK)
		{
			case ALINE_RIGHT:
				break;
	
			case ALINE_CENTER:
					right = right - (right - left - sizeofstring) / 2;
				break;
	
			case ALINE_LEFT:
				right = left + sizeofstring;
				right = right + FontSpace;
				break;
		}//switch

		SetCursorX(right);		
	}
	
	SetCursorY(top);
	Puts(str);
}	//PutText

void PutTextP(unsigned short left, unsigned short top, unsigned short right, unsigned short bottom, const char *str, unsigned char style)
{
	char 				strP[100+1];
	
	StrnCpy(strP, str, StrnLen(str,100));
	strP[100] = 0;	// terminate string
	PutText(left, top, right, bottom, strP, style);
}	//PutTextP

// void TextBox(unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, char *str, unsigned char style)
// {
// 	unsigned int 			r;

// 	if (style & BORDER_RECT)
// 	{
// 		r = (style & BORDER_BEVEL) >> 4;	// get bevel radius
// 		if (r)	// bevel
// 		{
// 			if (style & BORDER_FILL)
// 				BevelFill(x1+1, y1+1, x2-1, y2-1, r, BkColor);
// 			
// 			Bevel(x1, y1, x2, y2, r, FgColor);
// 		}
// 		else	// rect
// 		{
// 			if (style & BORDER_FILL)
// 				RectangleFill(x1+1, y1+1, x2-1, y2-1, BkColor);

// 			Rectangle(x1, y1, x2, y2, FgColor);
// 		}
// 	}
// 	else if (style & BORDER_FILL)
// 	{
// 		RectangleFill(x1, y1, x2, y2, BkColor);
// 	}

// 	if ((style & BORDER_RECT) || (style & BORDER_FILL))
// 	    PutText(x1+1, y1+1, x2-1, y2-1, str, style & ALINE_MARK);
//     else
// 	    PutText(x1, y1, x2, y2, str, style & ALINE_MARK);
// }	//TextBox

// void TextBoxP(unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, char *str, unsigned char style)
// {
// 	char 		strP[100+1];
// 	
// 	StrnCpy(strP, str, StrnLen(str,100));
// 	strP[100] = 0;	// terminate string
// 	TextBox(x1, y1, x2, y2, strP, style);
// }	//TextBoxP

/////////////////////////////////////
void SetLetter(Lt_Mode L)
{
	peLETTER = L;
	
	if(L == P_LETTER)
		cursorX = GetMaxX();
	else
		cursorX = 0;
}	//SetLetter

void PutsPE(char *str)
{	
	unsigned char 	curnLet,	dumyLet;
	unsigned char 	Pstat, stat, Nstat;
	
	unsigned char		backFontSpace = FontSpace;
	char 						strTemp[50];
	unsigned char 	i = 0, j = 0;

 	while((curnLet = *str++))
	{ 		
 		switch(curnLet)
 		{	
 			case 0x81:				//peh
    		curnLet = 0xEA;
 				break;
 			case 0x8D:				//cheh
    		curnLet = 0xEB;
 				break;
 			case 0x8E:				//zheh
    		curnLet = 0xEE;
 				break;
 			case 0x90:				//geh
    		curnLet = 0xEF;
 				break;
 			case 0xE1:				//laa
    		nextLet = *str++;
    		
    		if(nextLet == 0xC7)
    			curnLet = 0xFB;
    		else
    		{
    			dumyLet = *str--;
    			curnLet = 0xE1;
    		}
 				break;
 			case 0x3F:				//persian question mark
    		curnLet = 0xBF;
 				break;
 			case 0x3B:				//persian semicolon
    		curnLet = 0x1F;
 				break;
 			case 0x2C:				//persian comma
    		curnLet = 0x1D;
 				break;
 			case 0x2E:				//persian point
    		curnLet = 0x1C;
 				break; 				
		}//switch curnLet
		
		//--------------------------------
  	nextLet = *str++;
  	dumyLet	= *str--;
 	
 		switch(nextLet)
 		{	
 			case 0x81:				//peh
    		nextLet = 0xEA;
 				break;
 			case 0x8D:				//cheh
    		nextLet = 0xEB;
 				break;
 			case 0x8E:				//zheh
    		nextLet = 0xEE;
 				break;
 			case 0x90:				//geh
    		nextLet = 0xEF;
 				break;
 			case 0x3F:				//persian question mark
    		nextLet = 0xBF;
 				break;
 			case 0x3B:				//persian semicolon
    		nextLet = 0x1F;
 				break;
 			case 0x2C:				//persian comma
    		nextLet = 0x1D;
 				break;
 			case 0x2E:				//persian point
    		nextLet = 0x1C;
 				break; 			
		}//switch nextLet

		if(curnLet > 0xC0)
		{		
			if(prevLet > 0xC0)
				Pstat = (PROGMEM_read_b((PEmap[prevLet-0xC1][5])));		// 1: prevLet attach to curnLet -- 0: prevLet don't attach to curnLet
			else
				Pstat = 0;
				
			if(nextLet > 0xC0)
				Nstat = (PROGMEM_read_b((PEmap[nextLet-0xC1][4])));		// 1: nextLet attach to curnLet -- 0: nextLet don't attach to curnLet
			else
				Nstat = 0;
			
			//			Pstat	|	Nstat	|	stat
			//		 -------+-------+------
			//				0		|		0		|		0				curnLet, don't attach to prevLet and nextLet
			//				0		|		1		|		1				curnLet, don't attach to prevLet and attach to nextLet
			//				1		|		0		|		2				curnLet, attach to prevLet and don't attach to nextLet
			//				1		|		1		|		3				curnLet, attach to prevLet and nextLet
			
			stat = (Pstat<<1) | Nstat;		
			
			if(stat>1)	FontSpace = 0;
			else 				FontSpace = backFontSpace;
			
			PutCharPE(PROGMEM_read_b((PEmap[curnLet-0xC1][stat])));
			FontSpace = backFontSpace;
		}
		else	//Original 'curnLet' below 193 without (     )  -- for digits and symbols!
		{
  		if((curnLet >= '0') && (curnLet <= '9'))		// 0 to 9
  		{
  			i++;
  			strTemp[i] 	 = curnLet;

  			if(!((nextLet >= '0') && (nextLet <= '9')))
				{
					for(j=i; j>0; j--)
						PutCharPE(strTemp[j]-0x20);

					i = 0;
				}
			}		
			else
				PutCharPE(curnLet);
		}
		
		prevLet = curnLet;
	}//while

 	prevLet = 0xFF;
}	//PutsPE

void PutCharPE(unsigned char c)
{
	unsigned char 			byte 					= 0;
	unsigned char 			bitoffset 		= 0;
	unsigned char 			maske 				= 0;
	unsigned short 			bcounter 			= 0;
	unsigned short 			bitsbischar 	= 0;
	unsigned short 			bytesbischar	= 0;
	unsigned short 			xPos,yPos;

	unsigned char 			Ccounter 			= 0;
	unsigned char 			xc 						= 0;
	unsigned char 			yc 						= 0;
	unsigned char 			sx 						= 0;
	unsigned char 			sy 						= 0;
	
	// get current character width
	charwidth	= PROGMEM_read_b(FontPointer[7+c-firstchar]);
	
	// line feed, goto next line
	if(c == '\n')
	{
		cursorX = GetMaxX();
		cursorY = cursorY + (unsigned int)FontHeight * FontYScale; 
		
		return;
	}

	// character out of range.
  if( (c < firstchar) || (c > lastchar) || (fontSize == 0)) 
  	return;

	// character is not in list.
	if (charwidth == 0)
		return;

	// sara thai font. line remain at last position
	if(FontFixed)
		charwidth = FontWidth;
	
	// line adjust
	if(((int)cursorX - charwidth * FontXScale) < 0)
  {
		cursorY = cursorY   + (unsigned int)FontHeight * FontYScale; 
		cursorX = GetMaxX() - charwidth * FontXScale;
	}

	// adjust cursor to current position - persian
	cursorX	-= charwidth * FontXScale + FontSpace;

	// calculate current character position on the table
	for(Ccounter = 0; Ccounter < c-firstchar; Ccounter++)
		bitsbischar += (PROGMEM_read_b(FontPointer[Ccounter+7]));	//c0_width +...+ cn_width


	bitsbischar 		*= FontHeight;										//c_widths * FH
	bitsbischar 		+= ((lastchar-firstchar)+7)*8;		//plus font headers
	bytesbischar 		 = bitsbischar/8;									//
	bitoffset 			 = bitsbischar % 8;								//
	maske						 = bitoffset % 8;									//

	// draw character
	for(xc = 0; xc < charwidth; xc++)
	{
		for(yc = 0; yc < FontHeight; yc++)
		{
			if(maske > 7)
			{
				maske = 0;
				bcounter+=1;
			}
			byte = PROGMEM_read_b(FontPointer[bytesbischar + bcounter + 1]);

			xPos = (unsigned short)xc*FontXScale + cursorX;
			
			if(Horizontal)	//128x64
			{
				if(Reverse)
					yPos = (((unsigned short)FontHeight-yc)*FontYScale + cursorY);
				else
					yPos = (((unsigned short)yc)*FontYScale + cursorY);     
			}
			else						//64x128
			{
				if(Reverse)
					yPos = (((unsigned short)FontHeight-yc)*FontYScale + cursorY);
				else
					yPos = (((unsigned short)yc)*FontYScale + cursorY);
				
			}	//Horizontal

			for(sx = 0; sx < FontXScale; sx++)
			{
				for(sy = 0; sy < FontYScale; sy++)
				{
					if(bit_is_one(byte,maske))
					{
						if(rot)		tftlcd_putPixel(yPos+sy, xPos+sx, FgColor);
						else			tftlcd_putPixel(xPos+sx, yPos+sy, FgColor);
					}
					else
					{
						if(NonTransparence)
						{
							if(rot)		tftlcd_putPixel(yPos+sy, xPos+sx, BkColor);
							else			tftlcd_putPixel(xPos+sx, yPos+sy, BkColor);
						}
					}
				}//for sy
			}//for sx
			
			maske++;
		}//for yc
	}//for xc	
}	//PutCharPE

void PutCharEN(unsigned char c)
{
	unsigned char 			byte 					= 0;
	unsigned char 			bitoffset 		= 0;
	unsigned char 			maske 				= 0;
	unsigned short 			bcounter 			= 0;
	unsigned short 			bitsbischar 	= 0;
	unsigned short 			bytesbischar	= 0;
	unsigned short 			xPos,yPos;
	
	unsigned char 			Ccounter 			= 0;
	unsigned char 			xc						= 0;
	unsigned char 			yc						= 0;
	unsigned char 			sx 						= 0;
	unsigned char 			sy 						= 0;

	// get current character width
	charwidth	= PROGMEM_read_b(FontPointer[7+c-firstchar]);
	
	// line feed, goto next line
	if(c == '\n')
	{
		cursorX = 0;
		cursorY = cursorY + (unsigned int)FontHeight * FontYScale; 
		
		return;
	}

	// character out of range.
  if( (c < firstchar) || (c > lastchar) || (fontSize == 0)) 
  	return;

	// character is not in list.
	if (charwidth == 0)
		return;

	// sara thai font. line remain at last position
	if(FontFixed)
		charwidth = FontWidth;
	else
	{
		//english special fonts!
		if (((c >= 0xd4) && (c <= 0xda)) || 
	  	  ((c >= 0xe7) && (c <= 0xec)) ||
				 (c == 0xd1))
		{
			cursorX = cursorX - charwidth * FontXScale;
		}
	}

	// fixed width for digit
 	if((c >= '0') && (c <= '9'))		// english : 0 to 9
		charwidth = PROGMEM_read_b(FontPointer[7+'0'-firstchar]);		//width reference = ZERO
	
	// line adjust
	if((cursorX + charwidth * FontXScale) > GetMaxX())
	{
		cursorY = cursorY + (unsigned int)FontHeight * FontYScale; 
		cursorX = 0;
	}

	// calculate current character position on the table
	for(Ccounter = 0; Ccounter < c-firstchar; Ccounter++)
		bitsbischar += (PROGMEM_read_b(FontPointer[Ccounter+7]));	//c0_width +...+ cn_width


	bitsbischar 		*= FontHeight;										//c_widths * FH
	bitsbischar 		+= ((lastchar-firstchar)+7)*8;		//plus font headers
	bytesbischar 		 = bitsbischar/8;									//
	bitoffset 			 = bitsbischar % 8;								//
	maske						 = bitoffset % 8;									//

	// draw character
	for(xc = 0; xc < charwidth; xc++)
	{
		for(yc = 0; yc < FontHeight; yc++)
		{
			if(maske > 7)
			{
				maske = 0;
				bcounter += 1;
			}
			byte = PROGMEM_read_b(FontPointer[bytesbischar + bcounter + 1]);

			xPos = (unsigned short)xc*FontXScale + cursorX;
			
			if(Horizontal)	//128x64
			{
				if(Reverse)
					yPos = (((unsigned short)FontHeight-yc)*FontYScale + cursorY);
				else
					yPos = (((unsigned short)yc)*FontYScale + cursorY);     
			}
			else						//64x128
			{
				if(Reverse)
					yPos = (((unsigned short)FontHeight-yc)*FontYScale + cursorY);
				else
					yPos = (((unsigned short)yc)*FontYScale + cursorY);
				
			}	//Horizontal

			for(sx = 0; sx < FontXScale; sx++)
			{
				for(sy = 0; sy < FontYScale; sy++)
				{
					if(bit_is_one(byte, maske))
					{
						if(rot)		tftlcd_putPixel(yPos+sy, xPos+sx, FgColor);
						else			tftlcd_putPixel(xPos+sx, yPos+sy, FgColor);
					}
					else
					{
						if(NonTransparence)
						{
							if(rot)		tftlcd_putPixel(yPos+sy, xPos+sx, BkColor);
							else			tftlcd_putPixel(xPos+sx, yPos+sy, BkColor);
						}
					}
				}//for sy
			}//for sx
			
			maske++;
		}//for yc
	}//for xc
	
	// adjust cursor to next position - english
	cursorX	+= charwidth * FontXScale + FontSpace;
}	//PutCharEN

unsigned int CalcTextWidthEN(char *str)
{
	unsigned int 		strSize = 0;
	unsigned char 	c;

	while((c = *str++))
	{
		if(c == '\n')			continue;
		
		if(FontFixed)
			strSize += FontWidth * FontXScale;
		else
		{
		  if((c < firstchar) || (c > lastchar)) 
				charwidth = FontWidth;
			else
			{
				charwidth = PROGMEM_read_b(FontPointer[7+c-firstchar]);

				//english spesial fonts!
				if (((c >= 0xd4) && (c <= 0xda)) || 
		    		((c >= 0xe7) && (c <= 0xec)) ||
				  	 (c == 0xd1))

					charwidth = 0;
			}
			
			strSize += charwidth * FontXScale;
		}
		
		strSize += FontSpace;
	}//while
	
	return strSize ;
}	//CalcTextWidthEN

unsigned int CalcTextWidthPE(char *str)
{
	unsigned char 	curnLet,	dumyLet;
	unsigned char 	Pstat, stat, Nstat;
	
	unsigned char		backFontSpace = FontSpace;
	unsigned int 		strSize = 0;
	
 	while((curnLet = *str++))
	{ 		
		if(curnLet == '\n')			continue;
	
		if(FontFixed)
			strSize += FontWidth * FontXScale;
		else
		{
		 	if((curnLet < firstchar) || (curnLet > lastchar)) 
				charwidth = FontWidth;
			else
			{
		 		switch(curnLet)
		 		{	
		 			case 0x81:				//peh
		    		curnLet = 0xEA;
		 				break;
		 			case 0x8D:				//cheh
		    		curnLet = 0xEB;
		 				break;
		 			case 0x8E:				//zheh
		    		curnLet = 0xEE;
		 				break;
		 			case 0x90:				//geh
		    		curnLet = 0xEF;
		 				break;
		 			case 0xE1:				//laa
		    		nextLet = *str++;
		    		
		    		if(nextLet == 0xC7)
		    			curnLet = 0xFB;
		    		else
		    		{
		    			dumyLet = *str--;
		    			curnLet = 0xE1;
		    		}
		 				break;
		 			case 0x30:				//persian digits 0...9
					case 0x31:
					case 0x32:
					case 0x33:
					case 0x34:
					case 0x35:
					case 0x36:
					case 0x37:
					case 0x38:
					case 0x39:
						curnLet -= 0x20;
		 				break;
		 			case 0x3F:				//persian question mark
		    		curnLet = 0xBF;
		 				break;
		 			case 0x3B:				//persian semicolon
		    		curnLet = 0x1F;
		 				break;
		 			case 0x2C:				//persian comma
		    		curnLet = 0x1D;
		 				break;
		 			case 0x2E:				//persian point
		    		curnLet = 0x1C;
		 				break; 				
				}//switch curnLet
		
				//--------------------------------
		  	nextLet = *str++;
		  	dumyLet	= *str--;
		  	
		 		switch(nextLet)
		 		{	
		 			case 0x81:				//peh
		    		nextLet = 0xEA;
		 				break;
		 			case 0x8D:				//cheh
		    		nextLet = 0xEB;
		 				break;
		 			case 0x8E:				//zheh
		    		nextLet = 0xEE;
		 				break;
		 			case 0x90:				//geh
		    		nextLet = 0xEF;
		 				break;
		 			case 0x30:				//0...9
					case 0x31:
					case 0x32:
					case 0x33:
					case 0x34:
					case 0x35:
					case 0x36:
					case 0x37:
					case 0x38:
					case 0x39:
						nextLet -= 0x20;
		 				break; 			
		 			case 0x3F:				//persian question mark
		    		nextLet = 0xBF;
		 				break;
		 			case 0x3B:				//persian semicolon
		    		nextLet = 0x1F;
		 				break;
		 			case 0x2C:				//persian comma
		    		nextLet = 0x1D;
		 				break;
		 			case 0x2E:				//persian point
		    		nextLet = 0x1C;
		 				break; 				
				}//switch nextLet

				if(curnLet > 0xC0)
				{		
					if(prevLet > 0xC0)
						Pstat = (PROGMEM_read_b((PEmap[prevLet-0xC1][5])));
					else
						Pstat = 0;
						
					if(nextLet > 0xC0)
						Nstat = (PROGMEM_read_b((PEmap[nextLet-0xC1][4])));
					else
						Nstat = 0;
					
					stat = (Pstat<<1) | Nstat;		
					
					if(stat>1)	FontSpace = 0;
					else 				FontSpace = backFontSpace;
					////////////////
					dumyLet = PROGMEM_read_b((PEmap[curnLet-0xC1][stat]));
					
					charwidth = PROGMEM_read_b(FontPointer[7+dumyLet-firstchar]);
					strSize += charwidth * FontXScale;
					strSize += FontSpace;
					////////////////
					FontSpace = backFontSpace;
				}
				else
				{
					charwidth = PROGMEM_read_b(FontPointer[7+curnLet-firstchar]);
					strSize += charwidth * FontXScale;
					strSize += FontSpace;
				}
				
				prevLet = curnLet;
			}
		}
	}//while

 	prevLet = 0xFF;
	return strSize ;
}	//CalcTextWidthPE

size_t StrNlen (const char* s, size_t maxlen)
{
	size_t len = 0;

  while ((len <= maxlen) && (*s))
  {
  	s++;
    len++;
  }

  return len;
} //StrNlen
