#include "stm32f4xx.h"
#include "ili9325.h"
#include "font8x16.h"
#include "delay.h"

/*
#define SYS_CLK 168

void delay_us(uint32_t us)     // us is microsecond time
{
	int i = 0, n = 0;
	n = (t * SYS_CLK / 5);
	for (i = 0; i <= n; i++);
}
*/

uint16_t x_text = 0, y_text = 0;

/**********************************************************************/
void tftlcd_writeIndex(uint16_t cmd)	 
{
	LCD_CS(0);
	LCD_RS(0);
	LCD_RD(1);
	LCD_WR(0);
	GPIO_Write(LCD_DataPort, cmd);
	LCD_WR(1);
	LCD_CS(1);
}
/**********************************************************************/
void tftlcd_writeData(uint16_t data)
{
	LCD_CS(0);
	LCD_RS(1);
	LCD_RD(1);
	LCD_WR(0);
	GPIO_Write(LCD_DataPort, data);
	LCD_WR(1);
	LCD_CS(1);
}
/**********************************************************************/
// uint16_t tftlcd_readIndex(uint16_t cmd)
// {
// 	LCD_CS(0);
// 	LCD_RS(0);
// 	LCD_RD(0);
// 	LCD_WR(1);
// 	GPIO_Write(LCD_DataPort, cmd);
// 	LCD_RD(1);
// 	LCD_CS(1);
// 	return 0; //!
// }
// /**********************************************************************/
// uint16_t tftlcd_readData(void)
// {
// 	uint16_t gRamData;
// 	LCD_CS(0);
// 	LCD_RS(1);
// 	LCD_RD(0);
// 	LCD_WR(1);
// 	gRamData = GPIO_ReadInputData(LCD_DataPort);
// 	LCD_RD(1);
// 	LCD_CS(1);
// 	return gRamData;
// }
/**********************************************************************/
static __inline void tftlcd_writeReg(uint16_t cmd, uint16_t data)
{
	tftlcd_writeIndex(cmd);
	tftlcd_writeData(data);
}
/**********************************************************************/
// static __inline uint16_t tftlcd_readReg(uint16_t cmd)
// {
// 	tftlcd_writeIndex(cmd);
// 	return tftlcd_readData();
// }
/**********************************************************************/
void tftlcd_gotoxy(uint16_t x, uint16_t y)
{
	if ((int16_t)x < 0) x = 0;
	if ((int16_t)y < 0) y = 0;
#ifdef PORTRAIT
	if (x > 239) x = 239;
	if (y > 319) y = 319;
#endif
#ifdef LANDSCAPE
	if (x > 319) x = 319;
	if (y > 239) y = 239;
#endif
	tftlcd_writeReg(0x20, x);
	tftlcd_writeReg(0x21, y);
} 
/**********************************************************************/
void tftlcd_setArea(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	
	if ((int16_t)x1 < 0) x1 = 0;
	if ((int16_t)y1 < 0) y1 = 0;
	if ((int16_t)x2 < 0) x1 = 0;
	if ((int16_t)y2 < 0) y1 = 0;
#ifdef PORTRAIT
	if (x1 > 239) x1 = 239;
	if (y1 > 319) y1 = 319;
	if (x2 > 239) x2 = 239;
	if (y2 > 319) y2 = 319;
#endif
#ifdef LANDSCAPE
	if (x1 > 319) x1 = 319;
	if (y1 > 239) y1 = 239;
	if (x2 > 319) x2 = 319;
	if (y2 > 239) y2 = 239;
#endif
	
#ifdef PORTRAIT
	tftlcd_writeReg(0x0050, x1);
	tftlcd_writeReg(0x0051, x2);
	tftlcd_writeReg(0x0052, y1);
	tftlcd_writeReg(0x0053, y2);
	tftlcd_writeReg(0x0020, x1);
	tftlcd_writeReg(0x0021, y1);
#endif
#ifdef LANDSCAPE
	tftlcd_writeReg(0x0052, x1);
	tftlcd_writeReg(0x0053, x2);
	tftlcd_writeReg(0x0050, y1);
	tftlcd_writeReg(0x0051, y2);
	tftlcd_writeReg(0x0021, x1);
	tftlcd_writeReg(0x0020, y1);
#endif
}
/**********************************************************************/
void tftlcd_clearScreen(uint32_t bkgColor)
{
	uint32_t counter;

	tftlcd_setArea(0, 0, MaxX, MaxY);
	tftlcd_writeIndex(0x0022);
	for (counter = 0; counter < 76800; counter++)
		tftlcd_writeData(bkgColor);
}
/**********************************************************************/
void tftlcd_backlightOn(void)
{
	LCD_Backlight(1);
}
/**********************************************************************/
void tftlcd_backlightOff(void)
{
	LCD_Backlight(0);
}
/**********************************************************************/
void tftlcd_init(void)
{
	// Reset controller
	LCD_Reset(1);
	delay_us(100);
	LCD_Reset(0);
	delay_us(2000);
	LCD_Reset(1);
	delay_us(2000);
	// ----------Start Initial Sequence ----------//
	tftlcd_writeReg(0x00E5, 0x78F0);   // Set internal timing				// !!!!!!!!!!! //
	
//	tftlcd_writeReg(0x00E5, 0x8000);   // Set internal timing				// !!!!!!!!!!! //

//	tftlcd_writeReg(0x0000, 0x0001);   // Start Oscillator					// !!!!!!!!!!! //
//	delay_us(1000);

	tftlcd_writeReg(0x0001, 0x0100);   // set SS and SM bit	  *( 0 for invert x)
	tftlcd_writeReg(0x0002, 0x0700);   // set 1 line inversion
#ifdef PORTRAIT
	tftlcd_writeReg(0x0003, 0x1030);   // set GRAM write direction and BGR = 0, 64K colors, 1 transfers/pixel.
#endif
#ifdef LANDSCAPE
	tftlcd_writeReg(0x0003, 0x1038);   // TODO: change landscape mode origin
#endif
	tftlcd_writeReg(0x0004, 0x0000);   // Resize register
	tftlcd_writeReg(0x0008, 0x0207);   // set the back porch and front porch
	tftlcd_writeReg(0x0009, 0x0000);   // set non-display area refresh cycle ISC[3:0]
	tftlcd_writeReg(0x000A, 0x0000);   // FMARK function
	tftlcd_writeReg(0x000C, 0x0000);   // RGB interface setting		// working values: 0x0001 , 0x0031 , 0x0000
	tftlcd_writeReg(0x000D, 0x0000);   // Frame marker Position
	tftlcd_writeReg(0x000F, 0x0000);   // RGB interface polarity
	//---------- Power On sequence ----------//
	tftlcd_writeReg(0x0010, 0x0000);   // SAP, BT[3:0], AP, DSTB, SLP, STB
	tftlcd_writeReg(0x0011, 0x0007);   // DC1[2:0], DC0[2:0], VC[2:0]
	tftlcd_writeReg(0x0012, 0x0000);   // VREG1OUT voltage
	tftlcd_writeReg(0x0013, 0x0000);   // VDV[4:0] for VCOM amplitude
	delay_us(20000);                   // Dis-charge capacitor power voltage
	tftlcd_writeReg(0x0010, 0x1090);   // SAP, BT[3:0], AP, DSTB, SLP, STB
	tftlcd_writeReg(0x0011, 0x0227);   // R11h=0x0221 at VCI=3.3V, DC1[2:0], DC0[2:0], VC[2:0]
	delay_us(50000);                   // delay 50ms
	tftlcd_writeReg(0x0012, 0x001F);   // External reference voltage= Vci;
	delay_us(50000);                   // delay 50ms
	tftlcd_writeReg(0x0013, 0x1500);   // R13=1200 when R12=009D; VDV[4:0] for VCOM amplitude
	tftlcd_writeReg(0x0029, 0x0027);   // R29=000C when R12=009D; VCM[5:0] for VCOMH
	tftlcd_writeReg(0x002B, 0x000D);   // Frame Rate = 91Hz
	delay_us(50000);                   // delay 50ms
	// ---------- Adjust the Gamma Curve ----------//
	tftlcd_writeReg(0x0030, 0x0000);
	tftlcd_writeReg(0x0031, 0x0707);
	tftlcd_writeReg(0x0032, 0x0307);
	tftlcd_writeReg(0x0035, 0x0200);
	tftlcd_writeReg(0x0036, 0x0008);
	tftlcd_writeReg(0x0037, 0x0004);
	tftlcd_writeReg(0x0038, 0x0000);
	tftlcd_writeReg(0x0039, 0x0707);
	tftlcd_writeReg(0x003C, 0x0002);
	tftlcd_writeReg(0x003D, 0x1D04);
#ifdef PORTRAIT
	tftlcd_writeReg(0x0060, 0xA700);     // Gate Scan Line
#endif
#ifdef LANDSCAPE
	tftlcd_writeReg(0x0060, 0x2700);
#endif
	tftlcd_writeReg(0x0061, 0x0001);     // NDL,VLE, REV
	tftlcd_writeReg(0x006A, 0x0000);     // set scrolling line
	//---------- Partial Display Control ----------//
	tftlcd_writeReg(0x0080, 0x0000);
	tftlcd_writeReg(0x0081, 0x0000);
	tftlcd_writeReg(0x0082, 0x0000);
	tftlcd_writeReg(0x0083, 0x0000);
	tftlcd_writeReg(0x0084, 0x0000);
	tftlcd_writeReg(0x0085, 0x0000);
	//---------- Panel Control ----------//
	tftlcd_writeReg(0x0090, 0x0010);
	tftlcd_writeReg(0x0092, 0x0600);
	tftlcd_writeReg(0x0007, 0x0133);      // 64K color and display ON
	tftlcd_clearScreen(WHITE);
}
/**********************************************************************/
void tftlcd_putPixel(uint16_t x, uint16_t y, uint16_t color)
{
	if ((int16_t)x < 0) x = 0;
	if ((int16_t)y < 0) y = 0;
#ifdef PORTRAIT
	if (x > 239) x = 239;
	if (y > 319) y = 319;
#endif
#ifdef LANDSCAPE
	if (x > 319) x = 319;
	if (y > 239) y = 239;
#endif
	tftlcd_setArea(x, y, x, y);
	tftlcd_writeReg(0x0022, color);
}
/**********************************************************************/
uint16_t tftlcd_getPixel(uint16_t x, uint16_t y)
{
	if ((int16_t)x < 0) x = 0;
	if ((int16_t)y < 0) y = 0;
#ifdef PORTRAIT
	if (x > 239) x = 239;
	if (y > 319) y = 319;
#endif
#ifdef LANDSCAPE
	if (x > 319) x = 319;
	if (y > 239) y = 239;
#endif

	return 0;
// TODO: tftlcd_getPixel
}
/**********************************************************************/
void tftlcd_line(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint16_t color)
{
	int16_t dy;
	int16_t dx;
	int16_t stepx, stepy;

	if ((int16_t)x0 < 0) x0 = 0;
	if ((int16_t)y0 < 0) y0 = 0;
	if ((int16_t)x1 < 0) x1 = 0;
	if ((int16_t)y1 < 0) y1 = 0;
#ifdef PORTRAIT
	if (x0 > 239) x0 = 239;
	if (y0 > 319) y0 = 319;
	if (x1 > 239) x1 = 239;
	if (y1 > 319) y1 = 319;
#endif
#ifdef LANDSCAPE
	if (x0 > 319) x0 = 319;
	if (y0 > 239) y0 = 239;
	if (x1 > 319) x1 = 319;
	if (y1 > 239) y1 = 239;
#endif

	dy = y1 - y0;
	dx = x1 - x0;

	if (dy < 0)
	{
		dy = -dy;
		stepy = -1;
	}
	else
	{
		stepy = 1;
	}
	if (dx < 0)
	{
		dx = -dx;
		stepx = -1;
	}
	else
	{
		stepx = 1;
	}
	dy <<= 1; 
	dx <<= 1; 
	tftlcd_putPixel(x0, y0, color);
	if (dx > dy) 
	{
		int16_t fraction = dy-  (dx >> 1); 
		while (x0 != x1)
		{
			if (fraction >= 0)
			{
				y0 += stepy;
				fraction -= dx; 
			}
			x0 += stepx;
			fraction += dy; 
			tftlcd_putPixel(x0, y0, color);
		}
	} 
	else 
	{
		int16_t fraction = dx - (dy >> 1);
		while (y0 != y1)
		{
			if (fraction >= 0)
			{
				x0 += stepx;
				fraction -= dy;
			}
			y0 += stepy;
			fraction += dx;
			tftlcd_putPixel(x0, y0, color);
		}
	}
}
/**********************************************************************/
void tftlcd_rect(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint16_t color, uint8_t fill)
{
	if ((int16_t)x0 < 0) x0 = 0;
	if ((int16_t)y0 < 0) y0 = 0;
	if ((int16_t)x1 < 0) x1 = 0;
	if ((int16_t)y1 < 0) y1 = 0;
#ifdef PORTRAIT
	if (x0 > 239) x0 = 239;
	if (y0 > 319) y0 = 319;
	if (x1 > 239) x1 = 239;
	if (y1 > 319) y1 = 319;
#endif
#ifdef LANDSCAPE
	if (x0 > 319) x0 = 319;
	if (y0 > 239) y0 = 239;
	if (x1 > 319) x1 = 319;
	if (y1 > 239) y1 = 239;
#endif

	if (fill)
	{
		uint32_t counter;
		tftlcd_setArea(x0, y0, x1, y1);
		tftlcd_writeIndex(0x0022);
		for (counter = 0; counter < ((y1 - y0 + 1) * (x1 - x0 + 1)); counter++)
			tftlcd_writeData(color);
		return;
	}
	tftlcd_line(x0, y0, x0, y1, color);
	tftlcd_line(x0, y1, x1, y1, color);
	tftlcd_line(x1, y1, x1, y0, color);
	tftlcd_line(x1, y0, x0, y0, color);
}
/**********************************************************************/
void tftlcd_circle(uint16_t cx, uint16_t cy, uint16_t rad, uint16_t color, uint8_t fill)
{
	uint16_t x, y;
	int16_t delta, tmp;
	x = 0;
	y = rad;
	delta = 3 - (rad << 1);

	while (y > x)
	{
		if (fill)
		{
			tftlcd_line(cx + x, cy + y, cx - x, cy + y, color);
			tftlcd_line(cx + x, cy - y, cx - x, cy - y, color);
			tftlcd_line(cx + y, cy + x, cx - y, cy + x, color);
			tftlcd_line(cx + y, cy - x, cx - y, cy - x, color);
		}
		else
		{
			tftlcd_putPixel(cx + x, cy + y, color);
			tftlcd_putPixel(cx - x, cy + y, color);
			tftlcd_putPixel(cx + x, cy - y, color);
			tftlcd_putPixel(cx - x, cy - y, color);
			tftlcd_putPixel(cx + y, cy + x, color);
			tftlcd_putPixel(cx - y, cy + x, color);
			tftlcd_putPixel(cx + y, cy - x, color);
			tftlcd_putPixel(cx - y, cy - x, color);
		}
		x++;
		if (delta>=0)
		{
			y--;
			tmp = (x << 2);
			tmp -= (y << 2);
			delta += (tmp + 10);
		}
		else
		{
			delta += ((x << 2) + 6);
		}
	}
}
/**********************************************************************/
void tftlcd_setCursor(uint16_t xPos, uint16_t yPos)
{
	
#ifdef PORTRAIT
	if (xPos > 29 || yPos > 19) 
	{
		yPos += xPos / 29;
		xPos %= 29;
	}
#endif
#ifdef LANDSCAPE
	if (xPos > 39 || yPos > 14) 
	{
		yPos += xPos / 39;
		xPos %= 39;
	}
#endif
	x_text = xPos;
	y_text = yPos;
	
	tftlcd_writeReg(0x0020, xPos);
  tftlcd_writeReg(0x0021, yPos);
}
/**********************************************************************/
void tftlcd_putChr(char character, uint16_t frgColor, uint16_t bkgColor)
{  
	int8_t vcount, hcount;
	char char_data;

	tftlcd_setArea(x_text * 8, y_text * 16, (x_text * 8) + 7, (y_text * 16) + 15);
	tftlcd_writeIndex(0x0022);
	for (hcount = 0; hcount < 16; hcount++)
	{
		char_data = font8x16[character - 0x20][hcount];

		for (vcount = 7; vcount > -1; vcount--)
		{
			if ((char_data&(1 << vcount)) >> vcount)
				tftlcd_writeData(frgColor);
			else
				tftlcd_writeData(bkgColor);
		}
	}

	#ifdef PORTRAIT
	if (++x_text > 29)
	{            
		x_text = 0;
		if (++y_text > 19)
			y_text = 0;
	}
#endif
#ifdef LANDSCAPE
	if (++x_text > 39)
	{            
		x_text = 0;
		if (++y_text > 14)
			y_text = 0;
	}
#endif
	
}
/**********************************************************************/
void tftlcd_putStr(char* string, uint16_t frgColor, uint16_t bkgColor)
{
	while (*string)
		tftlcd_putChr(*string++, frgColor, bkgColor);
}
/**********************************************************************/
void tftlcd_showBitmap16Bit(uint16_t x, uint16_t y, uint16_t xSize, uint16_t ySize, const uint16_t *bmp_array)
{
	uint32_t i;
	tftlcd_setArea(x, y, x + xSize - 1, y + ySize - 1);
	tftlcd_writeIndex(0x0022);

	for (i = 0; i < (xSize * ySize); i++)
	{
		tftlcd_writeData((uint16_t)*bmp_array++);
		//if (i % 240 == 0)
			//delay_ms(3);
	}
	
}
/**********************************************************************/
void tftlcd_showBitmap8Bit(uint16_t x, uint16_t y, uint16_t xSize, uint16_t ySize, const uint8_t *bmp_array)
{
	uint32_t i;
	uint16_t color;
	tftlcd_setArea(x, y, x + xSize - 1, y + ySize - 1);
	tftlcd_writeIndex(0x0022);

	for (i = 0; i < (xSize * ySize); i++)
	{
		color = bmp_array[i * 2 + 1] << 8;
		color |=  bmp_array[i * 2];
		tftlcd_writeData(color);
	}
}
/**********************************************************************/
void tftlcd_test(void)
{
#ifdef PORTRAIT
	tftlcd_rect(0, 0,   MaxX, 42,   RED,   1);
	tftlcd_rect(0, 43,  MaxX, 84,   GREEN, 1);
	tftlcd_rect(0, 85,  MaxX, 126,  BLUE,  1);
	tftlcd_rect(0, 127, MaxX, 168,  RED,   1);
	tftlcd_rect(0, 169, MaxX, 210,  GREEN, 1);
	tftlcd_rect(0, 211, MaxX, 252,  BLUE,  1);
	tftlcd_rect(0, 253, MaxX, 294,  RED,   1);
	tftlcd_rect(0, 295, MaxX, MaxY, GREEN, 1);
#endif
#ifdef LANDSCAPE
	tftlcd_rect(0, 0,   MaxX, 39,   RED,   1);
	tftlcd_rect(0, 40,  MaxX, 79,   GREEN, 1);
	tftlcd_rect(0, 80,  MaxX, 119,  BLUE,  1);
	tftlcd_rect(0, 120, MaxX, 159,  RED,   1);
	tftlcd_rect(0, 160, MaxX, 199,  GREEN, 1);
	tftlcd_rect(0, 200, MaxX, MaxY, BLUE,  1);
#endif
}
/**********************************************************************/

